import re
import os

from . import config
from .helpers import parseRecipeHeader
from .helpers import parseYamlDict
from .Module import Module
from .Defaults import Defaults


class Package:

    def __init__(self, name, version = None):
        self.name = name
        self.path = os.path.join(config.ali_prefix, 'alidist', name.lower() + '.sh')

        parsed = parseRecipeHeader(self.path)
        self.requires = parsed['requires'] if 'requires' in parsed.keys() else []
        self.devel = parsed['build_requires'] if 'build_requires' in parsed.keys() else []
        self.version = parsed['version'] 
        self.tag = parsed['tag'] if 'tag' in parsed else parsed['version']
        if version != None:
            self.module_version = version
            self.module_dep_versions = Module(name).deps_as_dict(version)
        else:
            self.module_version = None;
            self.module_dep_versions = {}    
        self.defaults = Defaults()

    def filter_deps(self, deps):
        """ List of deps with already applied arch and defaults filters """
        filtered = []
        # Filter on arch
        for dep in deps:
            filter = dep.split(':')
            if len(filter) == 2:
                x = re.match(filter[1], config.architecture)
                if x and filter[0]:
                    filtered.append(filter[0])
            else:
                filtered.append(dep)
        # Filter on disable from defaults
        if self.defaults is not None:
            return [x for x in filtered if x not in self.defaults.disable]
        return filtered

    def deps_with_versions(self):
        """ In addition to list of dependencies it provides correct versions from modulefile """
        dict = {}
        deps = self.filter_deps(self.requires)
        for dep in deps:
            if dep in self.module_dep_versions.keys():
                dict[dep] = self.module_dep_versions[dep]
            else:
                dict[dep] = 'from_system'  # TODO: rename dep into system RPM name
        return dict

    def get_extra_deps(self):
        """ Provide extran deps list coming from alidist recipe """
        with open(self.path) as recipe:
            content = recipe.read()
            found = re.search(r'cat ' + re.escape('> $INSTALLROOT/.rpm-extra-deps <<') + 'EoF\n.*?EoF', content, re.DOTALL)
            if found is not None:
                return found.group().split('\n')[1:-1]
        return []

    def get_devel_deps(self):
        deps = self.filter_deps(self.devel)
        if 'alibuild-recipe-tools' in deps:
            deps.remove('alibuild-recipe-tools')
        return deps

    def get_devel_deps_with_versions(self):
        dict = {}
        deps = self.get_devel_deps()
        for dep in deps:
            avail = Module(dep).versions()
            avail_filter = list(filter(lambda ver: ver.find('latest'), avail))
            if not avail:
                dict[dep] = 'from_system'
            else:
                dict[dep] = avail_filter[0]
        return dict



import os
import sys
import logging

from . import config
from .helpers import runSubprocess

class Repo:
    def __init__(self):
        if not runSubprocess(['createrepo', '-h'], failOnError=False) and not config.dry_run:
            logging.error('createrepo is not installed')
            sys.exit(1)

    def create(self):
        logging.info('Recreating repo in %s' % config.target_rpm_dir)
        command = ['createrepo', config.target_rpm_dir]
        if config.dry_run:
            print(*command)
        else:
            runSubprocess(command)



import os
ali_prefix = os.getcwd()
work_dir = os.path.dirname(os.path.realpath(__file__))
dry_run = False
target_rpm_dir = '/tmp/o2_rpms'
skip_deps = False
devel = False
force_build = False
log_level='INFO'
s3_bucket='s3://system-configuration/RPMS/'
release_tag='dev'
architecture='slc7_x86-64'
tag_version=False



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='f2r',
    version='1.5.1',
    author='Adam Wegrzynek',
    author_email='adam.wegrzynek@cern.ch',
    url='https://gitlab.cern.ch/AliceO2Group/flp-to-rpm',
    license='GPLv3',
    description='Produces RPMs for FLP packages out of aliBuild ouput',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities',
        'Environment :: Console',
        'Operating System :: Unix',
        'Development Status :: 4 - Beta'
    ],
    python_requires='>=3.6',
    install_requires=['PyYAML', 'coloredlogs', 's3cmd', 'alibuild'],
    scripts=['f2r'],
    include_package_data=True,
    package_data={
        'flp2rpm': ['after_remove_template.sh', 'after_install_template.sh', 'runtime.slc7.yaml', 'devel.slc7.yaml', 'runtime.slc8.yaml']
    }
)
